from typing import List, AnyStr
import math

class BasicChart(object):
    def __init__(self, name="default"):
        self.__name = name

    def _sanitize_list(self, l):
        if hasattr(l, 'tolist'):
            return l.tolist()
        return l

    def set_series_index(self, idx):
        self.__name = self.__name or "Series {}".format(idx)


    @property
    def x_ticks(self):
        return None

    @property
    def y_ticks(self):
        return None

    @property
    def data(self):
        return []

    def as_series(self):
        return {
            "name": self.__name,
            "data": self.data
        }


class Bar(BasicChart):
    def __init__(self, x: List, y: List, name: AnyStr=None):
        super(Bar, self).__init__(name=name)
        self.__x = self._sanitize_list(x)
        self.__y = self._sanitize_list(y)


    @classmethod
    def chart_type(cls):
        return "bar"

    @property
    def x_ticks(self):
        return self.__x

    @property
    def data(self):
        return self.__y


class Heatmap(BasicChart):
    def __init__(self, z: List, name: AnyStr=None):
        super(Heatmap, self).__init__(name=name)
        self.__x = []
        self.__y = []
        self.__z = []
        self.__tokenize_list(self._sanitize_list(z))


    def __tokenize_list(self, l):
        for q in l:
            if(len(q) == 2):
                (x,y),z = q
            elif(len(q) == 3):
                x,y,z = q
            else:
                raise ValueError("Heatmap only support in values in type of ((x,y),z) or (x,y,z)")
            if math.isnan(z): continue
            if x not in self.__x:
                self.__x.append(x)
            if y not in self.__y:
                self.__y.append(y)
            self.__z.append((x, y, z))
        self.__x = sorted(self.__x)
        self.__y = sorted(self.__y)
        self.__z = [(self.__x.index(x), self.__y.index(y), z) for x,y,z in self.__z]

    @classmethod
    def chart_type(cls):
        return "heatmap"

    @property
    def x_ticks(self):
        return self.__x

    @property
    def y_ticks(self):
        return self.__y

    @property
    def data(self):
        return self.__z


class MatrixHeatmap(Heatmap):
    def __init__(self, matrix, name: AnyStr=None):
        z = []
        for y in range(len(matrix)):
            for x in range(len(matrix[0])):
                z.append((x,y,matrix[y][x]))
        super(MatrixHeatmap, self).__init__(z,name)


class Scatterplot(BasicChart):
    def __init__(self, x: List, y: List, name: AnyStr=None):
        super(Scatterplot, self).__init__(name=name)
        self._x = self._sanitize_list(x)
        self._y = self._sanitize_list(y)

    @classmethod
    def chart_type(cls):
        return "scatter"

    @property
    def data(self):
        return list(zip(self._x, self._y))


class CategoricalScatterplot(Scatterplot):

    @property
    def x_axis(self):
        return sorted(list(set(self._x)))

    @property
    def y_axis(self):
        return sorted(list(set(self._y)))


    @property
    def data(self):
        x_ax = self.x_axis
        y_ax = self.y_axis
        x_i = list(map(lambda x: x_ax.index(x), self._x))
        y_i = list(map(lambda y: y_ax.index(y), self._y))
        return list(zip(x_i, y_i))





from cnvrg.helpers.apis_helper import post as apis_post, get as apis_get, credentials
from cnvrg.modules.cnvrg_files import CnvrgFiles
from cnvrg.modules.errors import CnvrgError
import cnvrg.helpers.config_helper as config_helper
import cnvrg.helpers.env_helper as env_helper
import cnvrg.helpers.args_helper as args_helper
import cnvrg.helpers.param_build_helper as param_build_helper
from cnvrg.helpers.url_builder_helper import url_join
import cnvrg.helpers.spawn_helper as spawn_helper
import json
class Project(CnvrgFiles):
    def __init__(self, project=None, project_url=None, working_dir=None):
        if project_url:
            owner_slug, project_slug = Project.get_owner_and_project_from_url(project_url)
        else:
            owner_slug, project_slug = param_build_helper.parse_params(project, param_build_helper.PROJECT, working_dir=working_dir)
        self.__owner = owner_slug or credentials.owner
        self.__project = project_slug

        in_dir = config_helper.is_in_dir(config_helper.CONFIG_TYPE_PROJECT, project_slug, working_dir)
        super(Project, self).__init__(in_dir=in_dir)
        if in_dir:
            self._set_working_dir(config_helper.find_config_dir(path=working_dir))
            self.in_dir = True
        elif not self.__project or not self.__owner:
            raise CnvrgError("Cant init project without params and outside project directory")

    def __fetch_config(self):
        resp = apis_get(self.get_base_url())
        return {"slug": self.slug, "owner": self.owner, **resp.get("result")}

    def is_git(self):
        resp = apis_get(url_join(self.get_base_url(), 'get_project'))
        if not resp.get("result"):
            return False
        result = json.loads(resp.get("result"))
        return result.get("git")


    @property
    def owner(self):
        return self.__owner

    @property
    def slug(self):
        return self.__project

    @property
    def git(self):
        config = self.get_config()
        if config:
            return config.get(":git") or False
        return self.__fetch_config().get("git")


    def get_git_commit(self):
        return spawn_helper.run_and_get_output("git rev-parse --verify HEAD")

    def get_git_branch(self):
        return spawn_helper.run_and_get_output("git rev-parse --abbrev-ref HEAD")

    def get_base_url(self):
        return "users/{owner}/projects/{project}".format(owner=self.__owner, project=self.__project)

    def _default_config(self):
        return {
            "project": self.__project,
            "owner": self.__owner,
            "commit": None
        }

    def web_url(self):
        return url_join(credentials.web_url(), self.__owner, 'projects', self.__project)

    def get_project_name(self):
        return self.__project

    def get_output_dir(self):
        return self.get_working_dir()

    def run_task(self, cmd, **kwargs):
        #if "grid" in kwargs: kwargs["grid"] = hyper_search_helper.load_hyper_search(kwargs["grid"])
        kwargs = {**env_helper.get_origin_job(), **kwargs}
        resp = apis_post(url_join(self.get_base_url(), "experiments"), data={"cmd": cmd, **kwargs})
        hyper_search = resp.get("hyper_search")
        if not hyper_search:
            error_msg = resp.get("error") or "Can't run experiment"
            raise CnvrgError(error_msg)
        return hyper_search


    def sync(self, **options):
        command = "cnvrg sync {args}".format(args=args_helper.args_to_string(options))
        spawn_helper.run_sync(command, print_output=True)


import requests
import json
import os
from cnvrg.modules.errors import CnvrgError
import cnvrg.helpers.auth_helper as auth_helper
import cnvrg.helpers.logger_helper as logger_helper
from cnvrg.helpers.url_builder_helper import url_join


JSON_HEADERS = {
    "Content-Type": "application/json"
}

def verify_logged_in():
    if not credentials.logged_in:
        raise CnvrgError("Not authenticated")

def __parse_resp(resp, **kwargs):
    try:
        r_j = resp.json()
        if r_j.get("status") and r_j.get("status") != 200 and r_j.get("message"):
            print(r_j.get("message"))
        return r_j
    except Exception as e:
        logger_helper.log_error(resp.text)
        logger_helper.log_bad_response(**kwargs)
        return {"status": 400, "error": str(e)}

def request(action, url, **kwargs):
    callee = getattr(session, action)
    try:
        resp = callee(url **kwargs)
        return __parse_resp(resp, url=url, **kwargs)
    except Exception as e:
        return {"status": 400, "error": str(e)}

def get(url, data=None):
    verify_logged_in()
    get_url = url_join(base_url, url)
    return request('get', get_url, timeout=420, params=data)

def post(url, data=None, files=None):
    verify_logged_in()
    get_url = url_join(base_url, url)
    return request('post', get_url, timeout=900, data=json.dumps(data), files=files)

def send_file(url, data=None, files=None):
    verify_logged_in()
    get_url = url_join(base_url, url)
    resp = requests.post(get_url, files=files, data=data, headers={"AUTH-TOKEN": credentials.token})
    return __parse_resp(resp, url=url, data=data)

def put(url, data=None):
    verify_logged_in()
    get_url = url_join(base_url, url)
    return request('put', get_url, timeout=900, data=json.dumps(data))

def download_file(url, dest=None):
    resp = requests.get(url)
    if dest:
        with open(dest, "w") as f:
            f.write(resp.text)
    return resp.text

def download_raw_file(url):
    resp = requests.get(url)
    return resp.content

def update_credentials():
    global credentials
    credentials = auth_helper.CnvrgCredentials()

credentials = auth_helper.CnvrgCredentials()
session = requests.session()
session.headers = {
    "AUTH-TOKEN": credentials.token,
    **JSON_HEADERS
}
base_url = url_join(credentials.api_url, "v1")

